#!/usr/bin/perl
# save_user.cgi
# Saves or creates a new user. If the changes require moving of the user's
# home directory or changing file ownerships, do that as well

BEGIN {
  require './school-lib.pl';
  require './school-users.pl';
};

use strict;
use vars qw(%in %text %config $logged $tb $cb $whatfailed);
use English;
use PasswdFile;
use PasswdLine;
use AliasFile;

$whatfailed = $text{whatfailed_addUserAct};

require './userTable.pl';

&ReadParse(\%in);

sub recommence {
    my ($message) = @_;

    &header($text{title_useradmin}, "", "user", 1, 0);
    hr;
    print "<b>$message</b>\n";
    br;
    my %params;
    $params{batchUsers} = $in{batchUsers};
    $params{regularize} = $in{regularize};
    if($params{regularize}) {
      $params{noPasswd}=1;
      $params{loginInvariant}=1;
      $params{title}=$text{rectify_title};
      $params{label}=$text{rectify_label};
    }
    userTable(%params);
    &footer("/", "index");
    exit(0);
}

if ($in{'cat'} eq "NONE" && !defined $in{'cat'}) {
	&recommence($text{addUserAct_choseCat});
}

if($in{cat} ne "EXA") {
  requireRole("admin");
}

if ($in{'pass1'} ne $in{'pass2'}) {
    &recommence($text{addUserAct_passDiff});
}
my $login;
if ($in{'regularize'}) {
    $login=checkUserForCleansing;
} else {
    $login=checkUserForCreation;
}
$|=1;
checkParams;

my $cat = $in{'cat'};
if ($in{'classe'} eq "" || $in{'classe'} eq "|") {
    if ($cat eq "ELE") {
	&recommence($text{addUserAct_class});
    }
    if ($cat eq "EXT") {
	&recommence($text{addUserAct_sponsor});
    }
}

my $classOrParr = $in{'classe'};
if($classOrParr =~ /^class\.(.*)/) {
  $classOrParr = $1;
} elsif($classOrParr =~ /^cat\.(.*)/) {
  $cat = $1;
}
if ($classOrParr eq "") {
  $classOrParr = $ENV{'REMOTE_USER'};
}

my $isExam=0;
if($cat eq "EXA") {
  $classOrParr = uc($login);
  $isExam=1;
}

# Check validity of category
checkCategory($cat, "ELE", "EXA", "GUE");

my $classe="[$cat ".&jmaToDaystring("date", \%in)." $classOrParr]";
my $alias=$in{'alias'};
my $firstName=$in{'firstName'};
my $lastName=$in{'lastName'};
my $ssn=$in{'ssn'};

if($ssn ne "" && $ssn !~ /^\d{11}/) {
    &recommence(text('addUserAct_badSsn', $ssn));
}

if ( ($in{'firstName'}.$in{'lastName'}) =~ /([()\t:,])/) {
    &error(&text('addUserAct_badchar',
		 &html_escape($1), $in{'firstName'}.$in{'lastName'}));
}

my $haveAlias;
if ($alias eq "" && !($in{suff_deb} && $in{suff_end})) {
    $alias = "$firstName.$lastName";
    $alias =~ tr//AAAAEEEEIIIIOOOOUUUUC/;
    $alias =~ tr//aaaaeeeeiiiioooouuuuc/;
    $alias =~ s/[^-a-zA-Z0-9_.]+/_/g;
    $alias =~ s/^\.*//g;
    $alias =~ s/\.*$//g;
    $alias =~ s/\.+/./g;
} else {
    $haveAlias = 1;
}

$alias =~ s/@.*$//g;
if ( $alias =~ /([^\w-_.])/) {
    &error(&text('addUserAct_badchar',
		 &html_escape($1), $alias));
}

my $aliasFile = Read AliasFile;
AliasFile::tallyUserdAliases;

my $oldOwner;
if (passwd->get($alias)) {
  if($haveAlias) {
    &error(&text('addUserAct_aliasIsLogin', $alias));
  } else {
    $alias = "";
  }
} elsif ($oldOwner=
	 $aliasFile->checkAlias($alias) ||
	 isAliasBusy($alias, $login)) {
  if ($haveAlias) {
    &error(&text('addUserAct_aliasClash', $alias, $oldOwner));
  } else {
    $alias = "";
  }
}

my $tmpFile="/tmp/userd$$";
if ($in{'regularize'}) {
    open (US, ">>/etc/userDB") ||
      &error(&text('error_openWrite', "/etc/userDB", $ERRNO));
} else {
    open(US, ">$tmpFile.us") ||
      &error(&text('error_openWrite', "$tmpFile.us", $ERRNO));
}

if($isExam && -d "/home/classes/cl".uc($login)."/commun") {
  &error(&text('error_noExamClass', $login));
}

my @users=();
if($in{suff_deb} && $in{suff_end}) {
    my $length = length($in{suff_deb});
    my $endLength = length($in{suff_end});
    my $maxLength = $length;
    if($endLength > $maxLength) {
	$maxLength = $endLength;
    }
    if($maxLength + length($login) > 8) {
	&error($text{'error_loginTooLong'});
    }
    for(my $i=$in{suff_deb}; $i <= $in{suff_end}; $i++) {
        my $user = sprintf("%s%0".$length."d", $login, $i);
	my $line="$user:N:$in{'cat'}:$lastName,$firstName:$classe:$alias";
	push(@users, "$user");
	print US "$line\n";
    }
} else {
    if(length($login) > 8) {
	&error($text{'error_loginTooLong'});
    }
    my $line="$login:N:$in{'cat'}:$lastName,$firstName:$classe:$alias:$ssn";
    push(@users, $login);
    print US "$line\n";
}

close(US);

if($in{'regularize'}) {
    &redirect("audit.cgi");
    exit(0);
}

if ($in{'pass1'} ne "") {
    umask(077);
    open (PW, ">$tmpFile.pw") ||
      &error(&text('error_openWrite', "$tmpFile.pw", $ERRNO));
    foreach my $user (@users) {
        my $line = "$user:$in{'pass1'}";
        print PW "$line\n";
    }
    close(PW);
}

&header($text{title_useradmin}, "", "user", 1, 0);
my $yarg="";
if($isExam) {
  $yarg="-y";
}
&watchProcess($config{'userd_location'}." $yarg $tmpFile.us", 1);
hr;

if($isExam) {
  print $text{addUserAct_waitPass}."<br>\n";
  require 'setExamPasswords.pl';
  changeExamPass(@users);
  hr;
}

printLink "addUser", $text{common_addOtherUser};
br;
printLink "editUser", $text{common_editThisUser}, {login=> $login};
br;
unlink("$tmpFile.us");
unlink("$tmpFile.pw");
&menu;
&footer("/", "index");
