# Displays table of class permissions

use strict;
use vars qw/%config %text %in $tb $cb $LLL_SECURITY_DIR/;
use English;
BEGIN {
    require 'school-lib.pl';
  require 'html.pl';
}
use GroupFile;
use ClassPerms;
use Carp qw/croak/;

require 'listfile.pl';
require 'killproc.pl';

my $SHARED_STATE = "sharedState.";
my $OLD_SHARED_STATE = "oldSharedState.";

my $PRIVATE_STATE = "privateState.";
my $OLD_PRIVATE_STATE = "oldPrivateState.";


my $stateCache;

sub getClassStates {

  if(!defined $stateCache) {
    $stateCache = { mail => getNamelist($LLL_SECURITY_DIR . "/mail/group.deny"),
		    all => getNamelist($LLL_SECURITY_DIR . "/all/group.deny"),
		    squid => getNamelist($LLL_SECURITY_DIR . "/squid/group.deny")};
  }
  return $stateCache;

}

sub updateClassStates {
  my ($states, $in) = @_;

  my $killMail=0;
  my $killAll=0;
  my $restartSquid=0;

  if(updateFileFromHtml($states->{mail}, $in, 'mailState', 1)) {
    $killMail=1;
  }
  if(updateFileFromHtml($states->{all}, $in, 'allState', 1)) {
    $restartSquid=1;
    $killAll=1;
  }
  if(updateFileFromHtml($states->{squid}, $in, 'squidState', 1)) {
    $restartSquid=1;
  }

  if($killMail) {
    killProcesses("/etc/lll/security/mail/group.deny",
		  { imapd => 1, ipop3d => 1 }, 1);
  }

  if($killAll) {
    killProcesses("/etc/lll/security/all/group.deny",
		  { imapd => 1, ipop3d => 1, smbd => 15 }, 1);
  }

  if($restartSquid) {
    reload('squid');
  }
}

sub handleClassPerms {
  my ($in) = @_;

  my %sharedPerms;
  my %privatePerms;

  if(ref($in) ne "HASH") {
    die "\$in should be a HASH reference";
  }

  foreach my $key (keys %$in) {
    if($key =~ /$OLD_PRIVATE_STATE(.*)/) {
      my $class = $1;
      my $nkey = $PRIVATE_STATE.$class;
      if($in->{$nkey} ne $in->{$key}) {
	$privatePerms{$class} = ($in->{$nkey} or "0");
      }
    } elsif($key =~ /$OLD_SHARED_STATE(.*)/) {
      my $class = $1;
      my $nkey = $SHARED_STATE.$class;
      if($in->{$nkey} ne $in->{$key}) {
	$sharedPerms{$class} = $in->{$nkey};
      }
    }
  }

 ClassPerms::apply(\%sharedPerms, \%privatePerms);
}

# Handle all class states
sub handleAllClassState {
  my ($in) = @_;
  if(ref($in) ne "HASH") {
    die "\$in should be a HASH reference";
  }

  my $state = getClassStates();
  updateClassStates($state, $in);
  handleClassPerms($in);
}

my $sharedChoices = [ [ ClassPerms::LOCKED , $text{classAccess_locked} ],
		      [ ClassPerms::READONLY, $text{classAccess_readonly} ],
		      [ ClassPerms::UNLOCKED, $text{classAccess_unlocked} ] ];


# DisplayLabels: if true, labels are displayed next to checkboxes
# DisplayClassName: if true, display class name in line
# IsMailEnabled: a hash of classes for which mail is disabled
sub classLine {
  $|=0;
  my ($displayLabels, $displayClassName, $isMailDisabled, $class) = @_;

  my $key = $config{classe_prefix}. $class;
  my $states = getClassStates();

  my $sharedState = ClassPerms::getSharedPerms($class);
  my $privateState = ClassPerms::getPrivatePerms($class);

#  &hidden("123456789url", $nextUrl);

  my $bg = { 'bgcolor'=> '#ff0000'};
  if($sharedState ne "invalid" && $privateState ne "invalid") {
    $bg = { $cb => ''};
  }

  tableRow(sub {
    if($displayClassName) {
      tableCell {
	printLink "editUser0", html_escape($class), { classe => $class };
      } {align=>'right'};
    };

    tableCell {
      if($displayLabels) {
	print $text{classAccess_labelGeneralAccess};
      }
      writeHtmlEntry($states->{all}, 'allState', $key, 1);
    } {align=>'center'};

    tableCell {
      if($displayLabels) {
	print $text{classAccess_labelShared};
      }
      hidden($OLD_SHARED_STATE.$class, $sharedState);
      printSelectionList($SHARED_STATE.$class, $sharedState, $sharedChoices);
    } {align=>'center'};

    tableCell {
      if($displayLabels) {
	print $text{classAccess_labelPrivate};
      }
      hidden($OLD_PRIVATE_STATE.$class, $privateState);
      printCheckbox($PRIVATE_STATE.$class, $privateState);
    } {align=>'center'};

    tableCell {
      if($displayLabels) {
	print $text{classAccess_labelMail};
      }
      writeHtmlEntry($states->{mail}, 'mailState', $key, 1);
    } {align=>'center'};

    tableCell {
      if($displayLabels) {
	print $text{classAccess_labelInternet};
      }
      writeHtmlEntry($states->{squid}, 'squidState', $key, 1);
    } {align=>'center'};

  },$bg);
}


sub classHeaderLine {
  my ($displayClassName) = @_;
  my @row = ( $text{classAccess_labelGeneralAccess},
	      $text{classAccess_labelShared},
	      $text{classAccess_labelPrivate},
	      $text{classAccess_labelMail},
	      $text{classAccess_labelInternet} );
  if($displayClassName) {
    unshift(@row, $text{classAccess_formTitle});
  }

  &tableRow([ @row ],{align=>"center"});
}

sub classButtonLine {
  my ($displayClassName) = @_;
  my $cols;
  if($displayClassName) {
    $cols = 6;
  } else {
    $cols = 5;
  }

  &tableRow(sub {submitLine($cols)});
}

1;
