#!/usr/bin/perl
# serviceRestart.cgi
# Presents a page allowing to restart various services

BEGIN {
  require './school-lib.pl';
  require './school-users.pl';
  require './dhcp.pl';
};

use strict;
use vars qw(%in %text %config $logged $tb $cb $whatfailed);

$whatfailed = $text{whatfailed_dhcpd};
&ReadParse(\%in);

requireRole('installateur');

sub ipCompare {
    my ($a, $b) = @_;

    my @a = split(/\./, $a);
    my @b = split(/\./, $b);
    for(my $i=0; $i<4; $i++) {
	my $r = $a[$i] <=> $b[$i];
	return $r if($r != 0);
    }
    return 0;
}


my ($defltPar, @entries0) = &loadDhcpConfig();
@entries0 = sort { ipCompare($a->[2], $b->[2]); } @entries0;

&header($text{title_boxadmin}, "", "dhcp", 1, 0);
my %hwTable;

my @entries;
foreach my $entry (@entries0) {
  my ($name, $hwAddr, $ipAddr) = @$entry;
  $hwAddr = canonHw($hwAddr);
  my $key = "changeMac-$hwAddr";
  if($hwTable{$hwAddr}) {
      push(@entries, [ $name, $hwAddr, $ipAddr . " Duplicate Mac $hwTable{$hwAddr}"]);
      next;
  }
  $hwTable{$hwAddr}="$name $ipAddr";

  push(@entries, [ $name, $hwAddr, sub {
	 printTextLine("$key",$ipAddr);
       } ]);
}

printLink("../dhcpd/index", $text{dhcp_moduleLink});

form "dhcpAct", sub {
    table ( {border=>1, cellspacing=>0},
	   [ {$tb},
	    [ {colspan=>3}, $text{boxAdmin_dhcp} ],
	    {$cb},
	     [ $text{dhcp_computer_name},$text{dhcp_hwaddr},$text{dhcp_ipaddr}],
	     @entries,
	     sub { submitLine(3); }
           ]);
};


my $ipAddr;
my $hwAddr;
my $bindingState;
my $name;
my $starts;
my $ends;

my @dynamicEntries;

open(LEASES, $config{'dhcp_leases_location'}) || &error("Could not open  file $config{'dhcp_leases_location'}");
while(<LEASES>) {
    s/\#.*//;
    my $line=$_;
    if($line =~ /^lease (.*) {/) {
	$ipAddr=$1;
	undef $hwAddr;
	undef $bindingState;
	undef $name;
    } elsif($line =~ /hardware ethernet (.*);/) {
	$hwAddr=$1;
    } elsif($line =~ /  binding state (.*);/) {
	$bindingState=$1;
    } elsif($line =~/client-hostname "(.*)";/) {
	$name=$1;
    } elsif($line =~/starts [0-9] (.*);/) {
	$starts=$1;
    } elsif($line =~/ends [0-9] (.*);/) {
	$ends=$1;
    } elsif($line =~/}\s*$/) {
	$hwAddr = canonHw($hwAddr);
	if(!defined $name) {
	    $name="";
	}
	my $key = "changeMac-$hwAddr";
	my $nameKey = "name-$hwAddr";
	my $nameLocal = $name;
	my $dynIpLocal = $ipAddr;
	push(@dynamicEntries, [$starts, $ends,
			       sub {
				   printTextLine($nameKey, $nameLocal);
			       },
			       $hwAddr,
			       $dynIpLocal,
			       sub {
				   printTextLine("$key", "");
			       }]);
    }
}
close(LEASES);

@dynamicEntries = sort { $b->[1] cmp $a->[1] } @dynamicEntries;
my @de;

foreach my $entry (@dynamicEntries) {
  my $hw=$entry->[3];
  next if($hwTable{$hw});
  $hwTable{$hw}=1;
  push(@de, $entry);
}

form "dhcpAct", sub {
    table ( {border=>1, cellspacing=>0},
	   [ {$tb},
	    [ {colspan=>6}, $text{boxAdmin_dhcp} ],
	    {$cb},
	     [ "starts", "ends", $text{dhcp_computer_name},$text{dhcp_hwaddr},$text{dhcp_dynip},$text{dhcp_ipaddr}],
	     @de,
	     sub { submitLine(6); }
           ]);
};

&menu;
&footer("/", "index");
