use strict;

&foreign_require("dhcpd", "dhcpd-lib.pl");

my %dhcpConfig = &foreign_config("dhcpd");
my $dhcpConfFile = $dhcpConfig{dhcpd_conf};

sub lockFile {
    &lock_file($dhcpConfFile);
}

sub commitFile {
    &flush_file_lines;
    &unlock_file($dhcpConfFile);
}

sub getDhcpIp {
    my ($host) = @_;

    my $ipAddr;
    my $fixedAddress = &foreign_call("dhcpd", "find", "fixed-address",
				     $host->{members});
    if(defined $fixedAddress) {
	$ipAddr = $fixedAddress->{value};
    }
    return $ipAddr;
}

sub loadDhcpConfig {
    my $conf = &foreign_call("dhcpd", "get_config");
    my ($par, $sub) = &foreign_call("dhcpd", "get_branch", "sub");
    my $mems = $par->{'members'};
    my $defltPar = $par;

    #  my @host = &foreign_call("dhcpd", "get_hosts");

    my @host;
    foreach my $h (&foreign_call("dhcpd", "find", "host", $mems)) {
	push(@host, [$par, $h]);
    }
    foreach my $s (&foreign_call("dhcpd", "find","subnet", $mems)) {
	foreach my $h (&foreign_call("dhcpd", "find",
				     "host", $s->{'members'})) {
	    push(@host, [$s, $h]);
	    #    $insubn{$h} = $s->{'index'};
	    $defltPar = $s;
	}
    }


    my @entries;
    foreach my $entry (@host) {
	my ($par, $h) = @$entry;
	my $name = $h->{value};
	my $mems = $h->{members};
	my $ipAddr=getDhcpIp($h);
	my $hardware = &foreign_call("dhcpd", "find", "hardware", $mems);
	if(defined $hardware && $hardware->{values}->[0] eq "ethernet") {
	    my $hwAddr = $hardware->{values}->[1];
	    push(@entries, [ $name, $hwAddr, $ipAddr, $h, $par ] );
	}
    }
    return ($defltPar, @entries);
}

sub canonHw {
    my ($hwAddr) = @_;
    $hwAddr =~ tr/A-Z/a-z/;
    $hwAddr =~ s/-/:/g;
    $hwAddr =~ s/^([0-9a-f]):/0\1:/;
    $hwAddr =~ s/:([0-9a-f]):/:0\1:/g;
    $hwAddr =~ s/:([0-9a-f])$/:0\1/;
    $hwAddr;
}

1;
