#!/usr/bin/perl
# save_user.cgi
# Saves or creates a new user. If the changes require moving of the user's
# home directory or changing file ownerships, do that as well

BEGIN {
    require './school-lib.pl';
}
require './dhcp.pl';

use strict;
use vars qw(%in %text %config $logged $tb $cb $whatfailed);
use PasswdFile;

$whatfailed = $text{whatfailed_dhcpAct};
requireRole("installateur");

sub checkIp {
    my ($ip) = @_;
    if ($ip !~ /^[\w\s\.\-,]+$/ ||
      $ip =~ /(^|[\s,])[-_]/ ||
	$ip =~ /\.([\s,\.]|$)/ ||
	$ip =~ /(^|[\s,])\d+\.[\d\.]*[a-z_]/i) {
	&error(&text('dhcp_invalidaddr', $ip));
    }
}

sub setField {
    my ($record, $indent, $field, @value) = @_;
    &foreign_call("dhcpd", "save_directive",
		  $record, $field,
		  [ { 'name' => $field,
		      'values' => [ @value ] } ], $indent);
}

&ReadParse(\%in);

lockFile();
my ($defltPar, @host) = &loadDhcpConfig();

my %ipTable;

foreach my $h (@host) {
    my ($name, $hwAddr, $ipAddr, $host, $par) = @$h;
    $hwAddr = canonHw($hwAddr);
    my $key = "changeMac-$hwAddr";
    my $newIp = $in{$key};
    if(defined $newIp) {
	if($newIp ne "" && $newIp ne $ipAddr) {
	    checkIp($newIp);
	    setField($host, 0, 'fixed-address', $newIp);
	    foreign_call("dhcpd", "save_directive",
			 $par, [ $host ], [ $host ], 1);
	    $ipAddr = $newIp;
	} elsif($newIp eq "") {
	    &foreign_call("dhcpd", "save_directive",
			  $par, [ $host ], [ ], 0);
	    undef $ipAddr;
	}
    }
    delete $in{$key};
    if(defined $ipAddr) {
	if(defined $ipTable{$ipAddr}) {
	    &error(&text('dhcp_duplicateip',
			 "$name/$hwAddr", $ipTable{$ipAddr}, $ipAddr));
	}
	$ipTable{$ipAddr} = "$name/$hwAddr";
    }
}


foreach my $key (keys %in) {
    next unless $key =~ /^changeMac-([0-9a-fA-F:-]*)$/;
    my $hwAddr = canonHw($1);
    my $name = $in{"name-$1"};
    my $newIp = $in{$key};
    next unless $newIp ne "";
    checkIp($newIp);
    if($name eq "") {
	&error(&text('dhcp_noname', $hwAddr));
    }
    if($name =~ /([^A-Za-z0-9_-])/) {
      my $badChar=html_escape($1);
      &error(&text('dhcp_name', $name, $badChar));
    }
    my $host = { name => 'host', type=>1, members=>[], values => [$name] };
    setField($host, 0, 'hardware', 'ethernet', $hwAddr);
    setField($host, 0, 'fixed-address', $newIp);
    &foreign_call("dhcpd", "save_directive", $defltPar, [ ], [ $host ], 0);

    if(defined $newIp) {
	if(defined $ipTable{$newIp}) {
	    &error(&text('dhcp_duplicateip',
			 "$name/$hwAddr", $ipTable{$newIp}, $newIp));
	}
	$ipTable{$newIp} = "$name/$hwAddr";
    }
}

commitFile();
restart($config{'dhcp_service'});
&redirect('dhcp.cgi');
