#!/usr/bin/perl

use English;

BEGIN {
  require './school-lib.pl';
  require './school-users.pl';
};


use strict;
use vars qw(%in %text %config $logged $tb $cb $whatfailed);
use PasswdFile;
use GroupFile;

$whatfailed=$text{whatfailed_editDir};
requireRole "titulaire";
&ReadParse(\%in);

my ($login, $userdbLine, $passwdLine) = checkUserForStudent;
my ($dir, $dirName) = parseDirName($passwdLine, $in{dirName});
if (! -d $dir) {
  &error($text{error_hackAttempt}." $dir is not a directory");
}

opendir(DIR, $dir) ||
  &error(&text('error_openDir', html_escape($dir), $ERRNO));

&header($text{title_useradmin}, "", "user#editfile", 1, 0);
form "editDirectoryAct",
  sub {
    table {$cb=>"", border=>1},
	sub {
	  my $i;
	  &tableRow([{colspan=>6, align=>"center"},
		     &text('editDirectory_editDir',
			   &html_escape("~$login$dirName"))], {$tb});
	  &tableRow([$text{editDirectory_type},
		     $text{editDirectory_mode},
		     $text{editDirectory_user},
		     $text{editDirectory_group},
		     $text{editDirectory_filename},
		     $text{editDirectory_doEdit} ],
		    {$cb=>'', align=>"center"});

	  $i=0;

	  my $file;
	  my  $target;
	  while(defined ($file=readdir(DIR))) {
	    my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
		$atime,$mtime,$ctime,$blksize,$blocks)
	      = lstat("$dir/$file");

	    my $type;
	    if (-l _) {
	      $type = "Link";
	    } elsif (-d _) {
	      $type = "Dir";
	    } elsif (-f _){
	      $type = "File";
	    } elsif (-p _) {
	      $type = "Fifo";
	    } elsif (-b _) {
	      $type = "BlockDevice";
	    } elsif (-c _) {
	      $type = "CharacterDevice";
	    } else {
	      $type = "Special";
	    }

	    $mode &= 07777;

	    my $uline = passwd->byId($uid);
	    my $user = (defined $uline) ? $uline->user : $uid;

	    my $gline = groupDB->byId($gid);
	    my $group = (defined $gline) ? $gline->group : $gid;

	    my $omode = sprintf ("%o",$mode);
	    tableRow {
	      &tableCell($text{"editDirectory_type$type"},
			 {align=>"center", 'wrap'});

	      if ($file eq ".." || $type eq "Link") {
		&tableCell("$omode");
		&tableCell("$user");
		&tableCell("$group");
	      } else {
		tableCell {
		  printTextLine("mode$i", $omode,
				{size=>4, maxlength=>4}) };
		&hidden("oldMode$i", $omode);

		tableCell {
		  printTextLine("user$i", $user,
				{size=>8, maxlength=>8}) };
		&hidden("oldUser$i", $user);

		tableCell {
		  printTextLine("group$i", $group,
					  {size=>8, maxlength=>8}) };
		&hidden("oldGroup$i", $group);
		    };

	      if ($file eq "." || $file eq "..") {
		&tableCell("$file");
	      } else {
		tableCell {
			    printTextLine("name$i", $file, {size=>20}); };
		&hidden("oldName$i", $file);
	      }

	      my $nextf;
	      if ($file eq ".") {
		$nextf = $dirName;
	      } elsif ($file eq "..") {
		$nextf = "/$dirName";
		$nextf =~ s;/[^/]*$;; ;
	      } else {
		$nextf = "/$dirName/$file";
	      }
	      $nextf =~ s/\/\/*/\//g;
	      $nextf =~ s/^\///g;
	      $nextf = &html_escape($nextf);

	      tableCell {
		if ($type eq "Dir") {
		  printLink("editDirectory",
			    $text{editDirectory_edit},
			    {login=>$login, dirName=>$nextf});
		} elsif ($type eq "File") {
		  printLink("editFile",
			    $text{editDirectory_edit},
			    {login=>$login, dirName=>$nextf});
		} elsif ($type eq "Link") {
		  $target = readlink("$dir/$file");
		  printTextLine("target$i", $target,
				{size=>40});
		  &hidden("oldTarget$i", $target);
		} else {
		  nbsp;
		  print "&nbsp;";
		}}, {align=>"center"};

	      if ($file ne "..") {
		$i++;
	      }
	    };
					      }
	  &tableRow([{colspan=>6},
		     "<b><br>$text{editDirectory_new}:"]);

	  &tableRow([ sub {
			printSelectionList("type$i", "",
					   [[ "Dir", "Dir"],
					    [ "File", "File"],
					    [ "Link", "Link"]]);
		      }, sub {
			printTextLine("mode$i", "", {size=>3, maxlength=>3});
		      }, sub {
			printTextLine("user$i", "", {size=>8, maxlength=>8});
		      }, sub {
			printTextLine("group$i", "",{size=>8, maxlength=>8});
		      }, sub {
			printTextLine("name$i", "", {size=>20});
		      }, {align=>"center"}, sub {
			printTextLine("target$i", "", {size=>40});
		      }]);

	  tableRow { submitLine(6, "Effectuer changement")};

	  &hidden('maxi', $i);
	  &hidden('dirName', $dirName);
	  &dumpLogin;
	}};

printLink("editUser", $text{userAdmin_editUser}, { login=>$login });
&menu;
&footer("/", "index");
