#!/usr/bin/perl
# save_user.cgi
# Saves or creates a new user. If the changes require moving of the user's
# home directory or changing file ownerships, do that as well

BEGIN {
  require './school-lib.pl';
  require './school-users.pl';
}

use strict;
use vars qw(%in %text %config $logged $tb $cb $whatfailed);
use PasswdFile;
use GroupFile;

$whatfailed = $text{whatfailed_editDirectoryAct};
requireRole("admin");
&ReadParse(\%in);

my ($login, $userdbLine, $passwdLine) = checkUserForStudent;
my $maxi = $in{'maxi'};
my ($dir, $dirName) = parseDirName($passwdLine, $in{dirName});

chdir $dir;
for (my $i=0; $i < $maxi + 1; $i++) {
    my $file = $in{"name$i"};
    if ($file =~ /\// || $file eq "..") {
	&logError($text{error_hackAttempt}."Dir $file is ..");
    }

    my $target;
    if ($i == $maxi && $file ne "" && $in{"oldname$i"} eq "") {
      my $type = $in{"type$i"};
      if ($type eq "Dir") {
	logMsg("mkdir $file");
	mkdir ($file, 0) ||
	  &error(&text('editDirectoryAct_createDir', $file, $!));
      } elsif ($type eq "File") {
	logMsg("touch $file");
	open FILE, ">$file" ||
	  &error(&text('editDirectoryAct_createFile', $file, $!));
	    close(FILE);
      }
      if ($type ne "Link") {
	$in{"target$i"} = "";
      }
    } elsif ($i == $maxi) {
      next;
    } elsif ($in{"name$i"} ne $in{"oldName$i"}) {
      my $oldname = $in{"oldName$i"};
      my $newname = $in{"name$i"};

      if ($newname eq "") {
	if (-d $oldname  && ! -l $oldname) {
	  logMsg("rmdir $oldname");
	  rmdir ($oldname) ||
	    &error(&text('editDirectoryAct_delDir', $oldname, $!));
	} else {
	  logMsg("rm $oldname");
	  unlink $oldname ||
	    &error(&text('editDirectoryAct_delFile', $oldname, $!));
	}
      } else {
	if (stat $newname) {
	  &logError(&text('editDirectoryAct_exists', $newname));
	}

	logMsg("mv $oldname $newname");
	rename ($oldname, $newname) ||
	  &error(&text('error_rename', $oldname, $newname, $ERRNO));
      }
    }

    if ($in{"target$i"} ne $in{"oldTarget$i"}) {
      unlink $file;
      $target = $in{"target$i"};
      logMsg("ln -s $target $file");
      symlink ($target, $file) ||
	&error(&text('editDirectoryAct_link', $file, $target, $!));
    }

    if ($in{"target$i"} ne "") {
      next;
    }

    if ($in{"mode$i"} ne $in{"oldMode$i"}) {
      logMsg("chmod ".$in{"mode$i"}." $file");
      chmod (oct($in{"mode$i"}) & 01777, $file);
    }

    if ($in{"user$i"} ne $in{"oldUser$i"} ||
	$in{"group$i"} ne $in{"oldGroup$i"} ) {

      my $user = $in{"user$i"};
      my $group = $in{"group$i"};

      my $uid;
      if ($user =~ /^[0-9]*$/) {
	$uid = $user;
      } else {
	my $usr = userDB->get($user);
	if(!defined $usr) {
	  &error(&text('editDirectory_noSuchUser', $user));
	}
	$uid = $usr->uid;
      }

      my $gid;
      if ($group =~ /^[0-9]*$/) {
	$gid = $group;
      } else {
	my $grp = groupDB->get($group);
	if(!defined $grp) {
	  &error(&text('editDirectory_noSuchGroup', $group));
	}
	$gid = $grp->gid;
      }

      if ($uid eq "") {
	&logError(&text('editDirectory_noSuchUser', $user));
      }

      if ($gid eq "") {
	&logError(&text('editDirectory_noSuchGroup', $group));
      }

      logMsg("chown $user.$group $file");
      chown ($uid, $gid, $file);
    }
}
&redirect("editDirectory.cgi?login=$login&dirName=$dirName");
