#!/usr/bin/perl
# editUser.cgi
# Saves or creates a new user. If the changes require moving of the user's
# home directory or changing file ownerships, do that as well

BEGIN {
  require './school-lib.pl',
  require './school-users.pl';
};

use strict;
use vars qw(%in %text %config $logged $tb $cb $whatfailed);
use PasswdFile;
use GroupFile;
use UserDBFile;

require './projects.pl';

use Note;

# Consultation uniquement
$logged = 1;

$whatfailed = $text{whatfailed_editProject};
&ReadParse(\%in);

my $owner;
my @vUsers;
my @vGroups;
my $fcmode;
my $title;
my $comment;

if ($in{isnew} ) {
  # Default value for owner
  $owner = $ENV{'REMOTE_USER'};
  push (@vUsers, $owner);
  push (@vGroups, "ens");
  $fcmode = 0775;
  $title = "$text{editProject_title}";
} else {
  validateProjectName($in{project});
  my ($dev, $ino, $mode, $nlink, $uid, $gid) =
    stat("/home/projects/$in{project}");
  my $uline = passwd->byId($uid);
  $owner = (defined $uline) ? $uline->user : $uid;

  getProjectShare($in{project});
  my $vUsers = foreign_call("samba", "getval", "valid users");
  @vUsers = split(/\s+/, $vUsers);
  foreach my $vUser (@vUsers) {
    if ($vUser =~ /^@(.*)/) {
      push (@vGroups, $1);
    }
  }
  $fcmode = oct(foreign_call("samba", "getval", "force create mode"));
  $comment = foreign_call("samba", "getval", "comment");
  $title = &text('editProject_titleName', html_escape($in{project}));
}

my @users;
foreach my $line (passwd->array) {
  if ($line->user =~ /\$/) {
    next;
  }
  if (!$line->isSystemUser()) {
    push (@users, $line->user);
  }
}
@users = sort @users;

my @groups;
foreach my $line (groupDB->array) {
  if($line->[2] >= 500) {
    push (@groups, $line->[0]);
  }
}
@groups = sort @groups;


my $memberWrite;
if ($fcmode & 020) {
  $memberWrite = 1;
} else {
  $memberWrite = 0;
}

my $worldReadable;
if ($fcmode & 004) {
  $worldReadable = 1;
} else {
  $worldReadable = 0;
}


&header($text{title_useradmin}, undef, "projects", 1, 0);

hr;

mainForm("saveProjectAct", $title,
	 { }, [
	       sub {
		 if ($in{isnew}) {
		   printLabeledTextLine "$text{general_name}", "project";
		 } else {
		   hidden("project", $in{project});
		 }
	       },

	       [ {align=>'right'}, "$text{general_comment}",
		 { colspan => 3, align=>'left'},
		 sub { printTextLine "comment", $comment,
		       { size => 40} } ],

	       sub { printLabeledTextLine "$text{general_owner}",
		       "owner", $owner,
		       {size => 8, maxlength=>8};
		     hidden("isnew", $in{isnew} ? 1 : 0);
		   },

	       sub { printLabeledMultiList "$text{editProject_memberUsers}",
		       "umembers",
		       \@users, \@vUsers;

		     printLabeledMultiList "$text{editProject_memberGroups}",
		       "gmembers",
		       \@groups, \@vGroups },

	       sub { printLabeledCheckbox
		       "$text{editProject_membersCanClobber}",
		       "memberwrite", $memberWrite;
		     printLabeledCheckbox
		       "$text{editProject_nonMembersCanRead}",
		       "worldreadable", $worldReadable;
		   },

	       [ {colspan=>2, align=>"center"},
		 sub { submit("$text{button_save}") },
		 sub { if(!$in{isnew}) {
		     submit("$text{button_delete}");}} ] ]);


printLink("projects", "$text{general_returnToList}");
hr;

&footer("/", "index");
