#!/usr/bin/perl
# editUser.cgi
# Saves or creates a new user. If the changes require moving of the user's
# home directory or changing file ownerships, do that as well

use Note;
use strict;

BEGIN {
  require 'school-lib.pl',
  require 'school-users.pl';
  require 'listfile.pl';
  require 'features.pl';
};

use vars qw(%in %text %config $logged $tb $cb $whatfailed);
require 'roles.pl';

# Consultation uniquement
$logged = 1;

$whatfailed = $text{whatfailed_editUser};
&ReadParse(\%in);

my ($login, $userdbLine, $passwdLine) = checkUserForModification($in{login});
&header($text{title_useradmin}, "", "user#modif", 1, 0);

hr;
print "<h1>".$text{userAdmin_edit}." <em>$login</em></h1>\n";

&printUserTable(0, sub {
    &printUser($userdbLine, 0, 0, 0);
});

require 'etc-passwd.pl';
require 'group.pl';

br();

my $isAdmin = haveRole('admin');

sub printActionButton {
    my ($title, $mode, $button) = @_;

    mainForm("deleteUserAct", $title,
	     [ sub {
		 &hidden("mode", $mode);
		 &dumpLogin;
		 &submitLine(1); } ] );
}

sub passwdBox {
  if(amSelfOrStudent($userdbLine)) {
    mainForm("passwdAct", $text{userAdmin_changePasswdTitle},
	     [[ {'wrap'}, $text{userAdmin_newPassword}, sub {
		 printPasswordInput("pass1", "") } ],
	      [ {'wrap'}, $text{userAdmin_newPasswordAgain}, sub {
		  &dumpLogin;
		  printPasswordInput("pass2", "")}],
	      \&submitLine]);
  } else {
    print "&nbsp;";
  }
}

sub ssnBox {
  if(canEditRights($userdbLine)) {
    mainForm("deleteUserAct", $text{userAdmin_setSsnTitle},
	     [[ {'wrap'}, $text{userAdmin_newSsn}, sub {
		  &dumpLogin;
		  &hidden("mode", "S");
		  printTextLine("ssn", "", {size=>11, maxlength=>11})}],
	      \&submitLine]);
  } else {
    print "&nbsp;";
  }
}

sub deleteBox {
  if ($isAdmin && !$userdbLine->isDeleted) {
    &printActionButton($text{editUser_remove1}, "D",
		       $text{editUser_remove2});
  }

  if(!$isAdmin && !$userdbLine->isStudent) {
    return;
  }
  if ($userdbLine->isInactive) {
    return unless haveRole('titulaire');
    &printActionButton($text{editUser_reactivate1}, "A",
		       $text{editUser_reactivate2});
  } else {
    return unless haveRole('stagiaire');
    &printActionButton($text{editUser_suspend1}, "H",
		       $text{editUser_suspend2});
  }
}

sub changeClass {
  my ($lastName,$firstName) = split(",", $userdbLine->name);
  my $alias = $userdbLine->mailAlias;
  my $canChange = 0;
  my @fields = ();
  if(amSelfOrStudent($userdbLine)) {
    push(@fields, [ $text{userTable_firstName}, sub {
		      printTextLine("firstName", $firstName, {size=>20})}],
	 [ $text{userTable_lastName}, sub {
	     printTextLine("lastName", $lastName, {size=>20})}],
	 [ $text{userdb_alias}, sub {
	     printTextLine("alias", $alias, {size=>20}) } ]);
    $canChange = 1;
  }

  if((canEditRights($userdbLine)) && !$userdbLine->isBatch) {
    my $catClass;
    if($userdbLine->isStudent) {
      my $classe = $userdbLine->class;
      $catClass = "class.".$classe;
    } else {
      $catClass = "cat.".$userdbLine->cat;
    }
    @fields = (@fields, [ $text{userdb_class}, sub {
			    printClassWidget("classe", $catClass, $isAdmin);
			  }]);
    $canChange=1;
  }

  if($canChange == 0) {
    print "&nbsp;";
    return;
  }

  @fields = (@fields, sub {
	       &hidden("mode", "M");
	       &dumpLogin;
	       submitLine(1)
	     });

  mainForm("deleteUserAct", $text{editUser_change}, \@fields);
}

sub greenButton {
    if(!amSelfOrStudentGb($userdbLine)) {
      print "&nbsp;";
      return;
    }
    form ("greenbuttonAct", sub {
	hidden (login => $login);
	table ([ { bgcolor=>"#00ff00"}, [ sub {submit("          ")} ]],
	       {border=>0, cellpadding=>2, cellspacing=>0})});
}

sub editFileBox {
    if(!amSelfOrStudent($userdbLine)) {
      print "&nbsp;";
      return;
    }
    form ("editDirectory", sub {
	&dumpLogin;
	submit($text{editUser_editFiles});
	&hidden("dirName", "/");
    });
}


sub renameUser {
    if(!haveRole('admin') || ! $userdbLine->isRenamable) {
      print "&nbsp;";
      return;
    }
    mainForm("renameUserAct", $text{userAdmin_renameUserTitle},
	     [[ {'wrap'}, $text{userAdmin_newName}, sub {
		  &dumpLogin;
		  printTextLine("newName", $in{'login'}, {size=>8, maxlength=>8})}],
	      \&submitLine]);
}


my $stat = $userdbLine->state;
my @notes = Note::getNotes($userdbLine->user);

sub notesBox {
  if (@notes > 0) {
    my $note = pop(@notes);
    $note->print();
    printLink("notesUser",
	      $text{editUser_notes},
	      { login => $login});
    hr; br;
  }
}

if($userdbLine->isDeleted) {
  table ({width=>"100%", cellpadding=>0, cellspacing=>0, noborder=>1},
	 [ {valign=>'top'},
	   [  \&notesBox ],
	   [ {align =>'left' }, \&deleteBox ]
	 ]);
} else {
  table ({width=>"100%", cellpadding=>5, cellspacing=>0, noborder=>1},
	 [ {valign=>'top'},
	   [{colspan=>3}, \&notesBox ],
	   [{align=>'left'},
	    $stat ne "H" ? \&passwdBox : "",
	    {align => "center" }, \&ssnBox,
	    {align =>'right' }, \&deleteBox ],

	   [{align=>"left"}, sub { featuresBox($login); },
	    {align=>"center"}, \&changeClass,
	    {align=>"center"}, sub { editFileBox; greenButton; renameUser } ]
	 ]);
}
hr;

if (@notes == 0) {
  printLink("notesUser",
	    $text{editUser_addNote},
	    { login => $login});
  hr;
}

&footer("/", "index");
