# Utility functions for accessing /etc/passwd
#

use English;

BEGIN {
    require './school-lib.pl';
}

use strict;
use vars qw(%in %text %config $logged $tb $cb $whatfailed);

use Lock;

sub getShell {
    my ($user) = @_;

    open (PASSWD, "<".config{'passwd_file'}) ||
	&error(&text('error_openRead', '/etc/passwd', $ERRNO));

    my $found=0;
    my @line;
    while(<PASSWD>) {
	chop;
	@line = split(":", $_);

	if($line[0] eq $user) {
	    $found=1;
	    last;
	}
    }
    close(PASSWD);
    if($found) {
	return $line[6];
    } else {
	return "";
    }
}

sub editShell {
    my ($user, $shell) = @_;
    &Lock::lock($config{'passwd_file'});
    open (PASSWDIN, "<".$config{'passwd_file'}) ||
	&error(&text('error_openRead', '/etc/passwd', $ERRNO));
    open (PASSWDOUT, ">".$config{'passwd_file'}.".new") ||
	&error(&text('error_openWrite', '/etc/passwd.new', $ERRNO));

    my $found=0;
    while(<PASSWDIN>) {
	chop;
	my @line = split(":", $_);

	if($line[0] eq $user) {
	    $line[6] = $shell;
	}
	my $line = join(":", @line);
	print PASSWDOUT "$line\n";
    }
    close(PASSWDIN);
    close(PASSWDOUT);
    rename $config{'passwd_file'}.".new", $config{'passwd_file'};
    &Lock::unlock($config{'passwd_file'});
}


sub printPasswdLines {
    my ($flag, $lines) = @_;

    table( {'border', $cb},
	  sub {
	      &tableRow([$text{etcPasswd_login},
			 $text{etcPasswd_uid},
			 $text{etcPasswd_gid},
			 $text{etcPasswd_finger},
			 $text{etcPasswd_homeDir},
			 $text{etcPasswd_loginShell},
			 $text{etcPasswd_erasedDate} ], {$tb => ""});
	      foreach my $line (@{$lines}) {
		  my $i=0;
		  tableRow {
		    foreach my $field (@{$line}) {
		      if ($i == 1) {
			$i++;
			next;
		      }
		      tableCell {
			if ($field eq "") {
			  nbsp;
			  return;
			}
			if ($i == 0 && $flag) {
			  printLink("rectify", $field,
				    { login => $field});
			} else {
			  print html_escape($field);
			}
		      };
		      $i++;
		    }
		    if ($i == 7) {
		      tableCell { nbsp };
		    }
		  } ({$cb => ''});
	  }});
}


1;
