#!/usr/bin/perl

use English;

BEGIN { require './school-lib.pl' };
use strict;
use vars qw(%in %text %config $logged $tb $cb $whatfailed
	   $table $tableName);


requireRole("installateur");
require './ip.pl';
$whatfailed = $text{whatfailed_firewallMain};
&ReadParse(\%in);


sub checkbox {
    my ($name) = @_;

    if ($in{$name}) {
	return "yes";
    } else {
	return "no";
    }
}

sub textarea {
    my ($name) = @_;

    my $result = $in{$name};

    $result =~ s/\s+/ /g;
    $result =~ s/^\s*//g;
    $result =~ s/\s*$//g;

    if($result =~ /([^.A-Za-z0-9:*+_ 	\/-])/) {
      my $char = $1;
      &error(&text('badChar', $char, $result));
    }
    return $result;
}

sub networks {
    my $result = &textarea(@_);
    my $line;
    my $ip;
    my $netmask;

    foreach my $line (split(" ", $result)) {
	$line =~ s/\#.*$//g;
	$line =~ s/^\s*//g;
	$line =~ s/\s*$//g;

	checkNetworkForValidity($line);
    }
    return $result;
}


sub network {
    my $result = &textline(@_);

    if ($result != "") {
	checkNetworkForValidity($result);
    }
    return $result;
}

sub ip_services {
#    my ($name) = @_;
#    my $result = $in{$name};
    my $result = &textarea(@_);

    foreach my $line (split(/ /, $result)) {
	$line =~ s/\#.*$//g;
	$line =~ s/^\s*//g;
	$line =~ s/\s*$//g;

	if($line =~ /^(self|[0-9.]*|\*):(.*)$/) {
	    my $ip = $1;
	    my $port = $2;
	    if($ip ne "self" && $ip ne "*") {
		checkNetworkForValidity($ip);
	    }
	    if($port ne "*") {
		checkPortForValidity($port);
	    }
	} else {
	    &error(&text('firewallMain_badService', $line));
	}
    }
    return $result;
}

sub ip_nat {
    my ($name) = @_;
    my $result = $in{$name};
    $result =~ s/[ 	]+//g;
    $result =~ s/\s+/ /g;
    $result =~ s/^\s*//g;
    $result =~ s/\s*$//g;

    foreach my $line (split(/ /, $result)) {
	$line =~ s/\#.*$//g;
	$line =~ s/^\s*//g;
	$line =~ s/\s*$//g;

	if($line =~ /^(([0-9.]*):)?([0-9]+)=>([0-9.]*):([0-9]+)$/) {
	    my ($oip, $oport, $ip, $dport) = ($2, $3, $4, $5);
	    if($oip) {
	      checkNetworkForValidity($oip);
	    }
	    checkNetworkForValidity($ip);
	    checkPortForValidity($oport);
	    checkPortForValidity($dport);
	} else {
	    &error(&text('firewallMain_badNat', $line));
	}
    }
    return $result;
}

sub ports {
    my $result = &textline(@_);
    checkPortSequenceForValidity($result);
    return $result;
}

sub port {
    my $result = &textline(@_);
    getWord($result);
    return $result;
}


sub devices {
    my ($name) = @_;
    my @result;
    my $key;

    foreach my $key (keys %in) {
	if ($key =~ /^$name\.(.*)/) {
	    push(@result, $1);
	}
    }

    return join(" ", @result);
}

sub textline {
    my ($name) = @_;
    return $in{$name};
}

sub link_checkbox {
    return checkbox(@_);
}


sub proxy {
    my ($name) = @_;
    my $i;
    my @accum=();

    for ($i=0; $i < 4; $i++) {
	push(@accum, $in{"${name}_$i"});
    }

    foreach my $idx (0, 1) {
	if($accum[$idx] == "") {
	    $accum[$idx] = "0/0";
	}
    }

    if($accum[3] == "") {
	$accum[3] = "0";
    }

    if ($accum[2] != "") {
	return join(",",@accum);
    } else {
	return "";
    }
}

sub serverList {
  my ($name, $value, $line) = @_;
  my $port = $line->[3];

  my $newValue = textarea($name.$port);
  my %scoreboard;
  foreach my $item (split(/\s+/, $newValue)) {
    $scoreboard{$item} = 1;
  }

  my @list;
  foreach my $item (split(/\s+/, $value)) {
    my @item = split(",", $item);
    if($item[0] =~ /\/0$/ && $item[2] == $port) {
      if(! exists $scoreboard{$item[1]}) {
	next;
      }
      delete $scoreboard{$item[1]};
    }
    push(@list, $item);
  }

  foreach my $item (keys %scoreboard) {
    push(@list, join(",", "0.0.0.0/0", $item, $port));
  }

  return join(" ", @list);
}

sub services {
  return undef;
}

require 'firewall.pl';

my %inFile;
readFile(\%inFile);

my %dict;
foreach my $line (@$table) {
    my $func = $line->[2];
    my $name=$line->[1];
    my $oldValue = $inFile{$name};
    my $value=$func->($name, $oldValue, $line);
    if(defined $value) {
      $dict{$name}=$value;
    }
}

writeFile(\%dict);
&printRedirect("firewallMain", {table => $tableName});
