#!/usr/bin/perl

BEGIN {
    $demo_allow = 1;
    require './school-lib.pl';
    require './logfile.pl';
};
use strict;
use vars qw(%in %text %config $logged $tb $cb);


&ReadParse(\%in);
require 'date.pl';

# Consultation uniquement
$logged = 1;

my $searchUser = $in{'user'};
my $searchUrl = $in{'url'};
my $searchClientHost = $in{'clientHost'};
my $denyOnly = $in{'denyOnly'};
my $recDelta = $in{'recent'};

my $filter="";
if ($searchUrl ne "") {
  $filter .= "| fgrep $searchUrl";
}

if ($searchUser ne "") {
  $filter .= "| fgrep $searchUser";
}

if ($searchClientHost ne "") {
  $filter .= "| fgrep $searchClientHost";
}


my @dates = getMinMaxDates \%in;
if ($dates[0] < time - 7200) {
    # only "titulaires" and "installators" have access to old dates
    requireRole("titulaire", "installateur");
}

my $studentsOnly;
my %student;
if (!haveRole("titulaire") && !haveRole("Stagiaire")) {
  $studentsOnly = 1;
  open(STUD, "</etc/students") ||
    &error(&text('error_openRead', "/etc/students", $ERRNO));
  while(<STUD>) {
    chop;
    $student{$_}=1;
  }
  close(STUD);
}


&header($text{title_logconsult}, "", "log", 1, 0);

my $opened = 0;
my $lineCtr;
sub tableHead {
  if ($opened == 1) {
    return;
  }
  print "<table border=1>";
  &tableRow([$text{log_date},
	     $text{log_hour},
	     $text{log_success},
	     $text{log_user},
	     $text{log_client},
	     $text{listSquidLog_url} ],
	    {$tb => ''}  );
  $opened = 1;
  $lineCtr = 0;
}

sub tableFoot {
  if ($opened == 0) {
    return;
  }
  print "</table><p>";
  $| = 1;
  $| = 0;
  $opened = 0;
}


logReadLoop {
  my ($line, $yearbase) = @_;

  my @line = split(/\s+/, $line);
  if ($line[0] eq "FATAL:") {
    return undef;
  }

  my $date = int($line[0]);

  my $user = $line[7];
  my $clientHost = $line[2];
  my $url = $line[6];

  if($studentsOnly && ! $student{$user}) {
    return undef;
  }

  if(($searchUser && $searchUser ne $user) ||
     ($searchClientHost && $searchClientHost ne $clientHost) ||
     ($searchUrl && index ($url, $searchUrl, 0) == -1) ||
     (defined $dates[0] && $line[0] < $dates[0]) ||
     (defined $dates[1] && $line[0] > $dates[1])) {
    return undef;
  }

  if ($denyOnly && $line[3] !~ 'DENIED') {
    return undef;
  }

  tableHead();
  &tableRow([{'nowrap'}, dateHeure($date), $line[3],
	     "<a href=\"editUser.cgi?login=$user\">$user</a>",
	     $clientHost,
	     {'wrap'}, "<a href=\"$url\">$url</a>" ], {$cb => ''});
  $lineCtr++;
  if ($lineCtr > 100) {
    tableFoot();
  }

  return undef;
} "/var/log/squid/access.log", $filter, @dates;

tableFoot();

print "&lt;End><p>";


&footer("/", "index");
