#!/usr/bin/perl
# save_user.cgi
# Saves or creates a new user. If the changes require moving of the user's
# home directory or changing file ownerships, do that as well

use English;
BEGIN {
    require './school-lib.pl';
}
use UserDBFile;
use GroupFile;
use Purge;
use POSIX;

use strict;
use vars qw(%in %text %config $logged $tb $cb $whatfailed);

$whatfailed = $text{whatfailed_purgeClasses};
requireRole("titulaire");
require 'school-users.pl';

my $purgeDir;
my $prefix = $config{classe_prefix};

sub makePurgeDir {
  if(defined $purgeDir) {
    return;
  }
  $purgeDir = "/home/classes/__PURGE__";
  if(!-d $purgeDir) {
    mkdir $purgeDir || die "Cannot make purge directory ($ERRNO)\n";
  }
  $purgeDir .= POSIX::strftime("/%Y%m%d-%H%M%S",localtime(time));
  if(!-d $purgeDir) {
    mkdir $purgeDir  || die "Cannot make purge directory ($ERRNO)\n";
  }
  $purgeDir .= '/';
}

&ReadParse(\%in);

my %class;

# Make inventory of userDB users
foreach my $line (userDB->array()) {
  next if(!$line->isStudent);
  $class{$line->class} = 1;
}

my @classes = getClasses(1);
foreach my $class (@classes) {
  next if($class{$class});
  rmdir "/home/eleves/$class";
}

foreach my $dir (glob "/home/classes/*") {
  next if(!-d $dir); # Not a directory
  next if($dir !~ /\/home\/classes\/($prefix(.*))/);
  my $group = $1;
  my $class = $2;
  next if($class{$class});
  unlink "$dir/commun";
  rmdir "$dir/commun";
  rmdir "$dir/membres";
  rmdir "$dir/templates";
  rmdir "$dir";
  if(-d $dir) {
    makePurgeDir;
    rename($dir, $purgeDir . $class);
  }
  groupDB->delete($group);
}

groupDB->foreach(sub {
  my ($gline) = @_;
  my $class = $gline->key;
  next if($class !~ /^$prefix[A-Z0-9_-]+$/);
  next if($class{$class});
  next if(-d "/home/classes/$class");
  groupDB->delete($class);
});

groupDB->write;

foreach my $dir (glob "/etc/lll/security/*") {
  foreach my $file (glob "$dir/group.*") {	
    Purge::purgeFileKeep($file, 022, groupDB->hash, ':');
  }
}


redirect("index.cgi");
