#!/usr/bin/perl

use English;
use strict;

BEGIN {
  require './school-lib.pl';
  require './school-users.pl';
}

use strict;
use vars qw(%in %text %config $logged $tb $cb $whatfailed);
use GroupFile;
use PasswdFile;

$whatfailed=$text{whatfailed_saveProject};

require "./projects.pl";


&ReadParse(\%in);

my @umembers=split(/\0/, $in{umembers});
my @members = @umembers;
foreach my $memb (split(/\0/, $in{gmembers})) {
  push (@members, "@".$memb);
}

validateProjectName($in{project});

my $owner=$in{owner};
my $line = passwd->get($owner);
if (!defined $line) {
    &error(&text('saveProjectAct_noUser', $owner));
}
my $uid = $line->[2];

my ($dir, $gid);
if ($in{isnew}) {
  # new creation

  # does the share already exist?
  my $found = foreign_call("samba", "get_share", $in{project});
  if ($found) {
    &error(&text('saveProjectAct_projectExists', $in{project}));
  }

  # does the group already exist?
  foreach my $elt (groupDB->array) {
    if ($elt->[0] eq $in{project}) {
      &error(&text('saveProjectAct_groupExists',$in{project}));
    }
  }

  $gid = groupDB->findFreeId;
  if (!defined $gid) {
    &error($text{saveProjectAct_noFreeGroup});
  }

  if(! -d "/home/projects") {
    &error(&text('projects_nodir'));
  }

  $dir = "/home/projects/".$in{project};

  (mkdir($dir, 0)) || &error(&text('projects_mkdir',$dir,$ERRNO));

  open(GROUP, ">>/etc/group");
  print GROUP "$in{project}:x:$gid:".join(",",@umembers)."\n";
  close(GROUP);

  foreign_call("samba", "setval", "directory", $dir);
  foreign_call("samba", "setval", "comment", "Project $in{project}");
  foreign_call("samba", "setval", "force group", $in{project});
} else {
  $dir = getProjectShare($in{project});

  my $line = groupDB->get($in{project});
  if (!defined $line) {
    &error(&text('saveProjectAct_noGroup', $in{project}));
  }
  $gid = $line->[2];

  if ($in{action} eq $text{button_delete}) {
    foreign_call("samba", "delete_share", $in{project});

    open(GROUP, "</etc/group");
    open(GOUT, ">/etc/group.$$");
    while(<GROUP>) {
      my @line = split(":");
      if ($line[0] eq $in{project}) {
	next;
      }
      print GOUT "$_";
    }
    close(GROUP);
    close(GOUT);
    rename "/etc/group.$$", "/etc/group";

    system("rm -rf $dir");
    &redirect("projects.cgi");
  }
}


foreign_call("samba", "setval", "valid users", join(" ", @members));
open(GROUP, "</etc/group");
open(GOUT, ">/etc/group.$$");
while(<GROUP>) {
    chomp;
    my @line = split(":");
    if ($line[0] eq $in{project}) {
	$line[3] = join(",",@umembers);
	$_ = join(":",@line);
    }
    print GOUT "$_\n";
}
close(GROUP);
close(GOUT);
rename "/etc/group.$$", "/etc/group";

# the directory modes
my $mode = 0640;
my $dirMode = 0750;
if ($in{memberwrite}) {
  $mode |= 020;
  $dirMode |= 020;
} else {
  $dirMode |= 01020; # sticky bit
}
if ($in{worldreadable}) {
  $mode |= 004;
  $dirMode |= 005;
}

foreign_call("samba", "setval", "force directory mode",
	     sprintf("0%o",$dirMode));
foreign_call("samba", "setval", "force create mode",
	     sprintf("0%o", $mode));

foreign_call("samba", "setval", "comment", $in{comment});
foreign_call("samba", "setval", "writable", "yes");

# save it
if ($in{isnew}) {
  foreign_call("samba", "create_share", $in{project});
} else {
  foreign_call("samba", "modify_share", $in{project}, $in{project});
}

chown $uid, $gid, $dir;


sub recursiveChmod {
  my ($name, $mode, $dirMode)=@_;
  my (@files);

  chmod $dirMode, $name;

  opendir(DIR, $name);
  my $f;
  while(defined($f = readdir(DIR))) {
    if ($f eq "." || $f eq "..") {
      next;
    }
    push(@files, $f);
  }

  foreach my $f (@files) {
    if (-d "$name/$f") {
      recursiveChmod("$name/$f", $mode, $dirMode);
    } else {
      chmod $mode, "$name/$f";
    }
  }
}

recursiveChmod($dir, $mode, $dirMode);

if($in{isnew}) {
  &redirect("projects.cgi");
} else {
  &redirect("editProject.cgi?project=$in{project}");
}
