use strict;
BEGIN {
    require 'school-lib.pl';
}
require 'userdb.pl';
require 'etc-passwd.pl';

use vars qw(%in %text %config $logged $tb $cb $whatfailed);
use PasswdFile;
use UserList;
use UserDBFile;
use UserDBLine;
use Keller;

my @categories;

if(isLllI()) {
    @categories = ( "ENS", "EXT", "FUN", "ORG", "SOP" );
} else {
    @categories = ( "ANC", "COM", "DRX", "ENS", "EXT", "FUN", "ORG", "PAR",
		    "POO", "SEC", "SER", "SOP", "SPO" );
}

sub commonChecks {
    my $login;

    if (scalar(@_) != 0) {
	$login = shift(@_);
    } else {
	# get the user
	$login = $in{login};
    }

    if ($login eq "") {
	&error($text{schoolUsers_empty});
    }

    if (length($login) > 8) {
	&error(&text('schoolUsers_tooLong', $login));
    }

    if ($login !~ /^([a-zA-Z][a-zA-Z0-9_]*)?(.?).*$/ || $2 ne "") {
	my $bad=$2;
	&error(&text('newClassAct_badChar',$bad,$login));
    }
    return $login;
}


sub checkUserForModificationInPasswd {
    my ($login, $modify) = @_;
    my $passwdLine = passwd->get($login);

    # he must exist in passwd
    if (! defined($login) ) {
      if ($modify) {
	&error(&text('schoolUsers_notIn',
		     $login, $config{'passwd_file'}));
      }
    } else {
      # he should not be a system user (uid < base_uid)
      if (defined $passwdLine && $passwdLine->isSystemUser) {
	&error(&text('schoolUsers_sysUser', $login));
      }
    }

    # keller users cannot be modified using webmin
    if (keller->contains($login)) {
      &error(&text('schoolUsers_theForce', $login));
    }
    return $passwdLine;
}

sub checkUserForModificationOrDestruction {
  my $flag = shift;
  my $login = commonChecks(@_);
  my $userdLine = userDB->get($login);
  if (!defined $userdLine) {
    &error(&text('schoolUsers_notIn', $login, $config{'userDB_file'}));
  }

  my $passwdLine = checkUserForModificationInPasswd($login, $flag);
  return ($login, $userdLine, $passwdLine);
}


sub checkUserForModification {
  return checkUserForModificationOrDestruction(1, @_);
}


sub checkUserForDestruction {
  return checkUserForModificationOrDestruction(1, @_);
}


sub checkUserForCreation {
    my $login = commonChecks;

    # he should not yet exist in userd
    if (defined userDB->get($login)) {
	&error(&text('schoolUsers_exists',
		     $login, $config{'userDB_file'}));
    }

    # he should not yet exist in passwd
    if (defined passwd->get($login) ) {
	&error(&text('schoolUsers_exists',$login, $config{'passwd_file'}));
    }

    # call the users by their own name FIXME
#    if (-f $config{'misnomers_users_file'} &&
#	userlist($config{'misnomers_users_file'},1)->contains($login)) {
#      &error(&text('schoolUsers_einstein', $login));
#    }

    # excluded users cannot be recreated using webmin
#    if (userlist($config{'excluded_users_file'},1)->contais($login)) {
#      &error(&text('schoolUsers_wagner', $login));
#    }
    return $login;
}

sub checkUserForCleansing {
    my $login = commonChecks;
    if (defined userDB->get($login)) {
	&error(&text('schoolUsers_exists', $login, $config{'userDB_file'}));
    }
    checkUserForModificationInPasswd($login, 1);
    return $login;
}

sub checkUserForStudent {
    my ($login, $userdbLine, $passwdLine) = checkUserForModification;
    if ($userdbLine->type ne "ELE") {
	&error($text{schoolUsers_studOnly});
    }
    return ($login, $userdbLine, $passwdLine);
}



sub checkParams {
  if ( ! ($in{'cat'} =~ /^[A-Z][A-Z][A-Z]$/) ) {
    &error($text{error_hackAttempt}. "bad cat $in{cat}");
  }

  for my $name ("lastName", "firstName", "classe", "alias") {
    if ( $in{$name} =~ /[:\n]/) {
      &error($text{error_hackAttempt}." colon in $name");
    }
  }
}

sub isExamClass {
  my ($class) = @_;

  my $classDir = "/home/classes/$config{classe_prefix}$class";
  if(-f "$classDir/commun") {
    return 1;
  } else {
    return 0;
  }
}

sub getClasses {
  my ($alsoExam) = @_;
  my @classes;
  my $file;
  opendir(DIR, "/home/eleves");
  while($file=readdir(DIR)) {
    if($file =~ /^\./) {
      next;
    }
    my $classDir = "/home/classes/$config{classe_prefix}$file";
    if(!-d $classDir) {
      next;
    }
    next if(!$alsoExam && -f "$classDir/commun");
    push(@classes, $file);
  }
  close(DIR);
  @classes = sort  @classes;
  return @classes;
}

sub catName {
  my ($cat) = @_;
  return $text{'userTable_cat'.ucfirst(lc($cat))};
}

sub categoryList {
  my (@in) = @_;

  my @out;
  foreach my $cat (@in) {
    push(@out, [ $cat, catName($cat) ]);
  }
  return @out;
}

sub categoryListSmall {
  return categoryList(@categories);
}

sub categoryListBig {
  return categoryList('ELE', @categories);
}

sub printClassWidget {
  my ($name,$choice, $showCategories)=@_;
  my @classesSel;

  push(@classesSel, [ "|", "[$text{schoolusers_choseclass}]" ]);
  foreach my $class (getClasses(0)) {
    push(@classesSel, [ 'class.'.$class, $class ]);
  }
  if($showCategories) {
    foreach my $cat (@categories) {
      push(@classesSel, ['cat.'.$cat, catName($cat) ]);
    }
  }
  printSelectionList($name, $choice, \@classesSel);
}

sub checkCategory {
  my ($cat, @list) = @_;
  my $found = 0;
  foreach my $cmp (@categories, @list) {
    if($cmp eq $cat) {
      $found = 1;
      last;	
    }
  }
  if(!$found) {
    &error(&text('general_noSuchCategory',$cat));
  }
}

1;
