# display the user table

require "./date.pl";
require "./school-users.pl";

BEGIN {
  require 'html.pl';
}

use strict;
use vars qw(%in %text %config);

my $catHeader = [ "NONE", "---$text{userTable_chooseCategory}---" ];
my $smallCategoryPairs = [ $catHeader, categoryListSmall() ];
my $bigCategoryPairs = [ $catHeader, categoryListBig() ];

sub userTable {
  my (%params) = @_;

  my $ops;
  if($params{allCategories}) {
    $ops = [[ "N", $text{userdb_create_modify} ],
	    [ "D", $text{button_delete} ],
	    [ "M", $text{userdb_modify} ]];
  } else {
    $ops = [[ "N", $text{userdb_create} ],
	    [ "D", $text{button_delete} ],
	    [ "M", $text{userdb_modify} ]];
  }

  if ($params{title} eq "") {
    if($params{studentOnly}) {
      $params{title} = $text{userTable_newStudent};
    } else {
      $params{title} = $text{userTable_newUser};
    }
  }

  if ($params{action} eq "") {
    $params{action} = "addUserAct";
  }

  if ($params{label} eq "") {
    if($params{studentOnly}) {
      $params{label} = $text{userTable_submitCreateStudent};
    } else {
      $params{label} = $text{userTable_submitCreateUser};
    }
  }

  hr;

  my @bigForm = ();

  if(defined $in{pattern}) {
    push(@bigForm,
	 sub {
	     printLabel($text{userdb_operation});
	     tableCell {
		 printSelectionList("oper", $in{oper}, $ops);
	   };
	     printLabel($text{userdb_formatName});
	     my $oldTemplate=$in{template};
	     if($oldTemplate eq "..") {
		 $oldTemplate = "";
	     }
	     tableCell (sub {
		 printTextLine("new_template", $oldTemplate,
			       {size=>30} )});
	     hidden(template => $in{template});
	 },
	 sub {
	     printLabel($text{userdb_lineFormat});
	     tableCell (sub {
		 printTextLine("pattern", $in{'pattern'},
			       {size=>60});
			}, {colspan=>3});
	 },
	 sub {
	     printLabel($text{userdb_defaultClass});
	     tableCell (sub {
		 printTextLine("defaultClass", $in{'defaultClass'},
			       {size=>30});
			}, {colspan => 3});
	 },
	 sub {
	     my $charsets = [[ "ISO-8859-1", "Latin 1" ],
			     [ "UTF-8", "UTF-8" ],
			     [ "MAC", "Macintosh" ]];
	     printLabel($text{userdb_charset});
	     tableCell(sub {
		 printSelectionList("charset", $in{charset}, $charsets);
		       }
		 );
	     printLabeledCheckbox($text{batch_ignoreFirstLine},
				  "ignoreFirstLine", $in{ignoreFirstLine});
	 }
    ); }

    my @line=();

    if ($params{batchUsers} ne "") {
      push(@line, {colspan=>2});
      push(@line, sub {
	     iprint("$text{general_start}:\n");
	     printTextLine("suff_deb", 1, {size=>3});
	     nbsp; nbsp; nbsp;
	     iprint("$text{general_end}:\n");
	     printTextLine("suff_end", 20, {size=>3});
	   });
      push(@line,  {colspan=>1});
    } else {
      push(@line, {align=>'right'});
      push(@line, $text{userdb_category});
      push(@line, {align=>'left'});
      if ($params{allCategories}) {
	  push(@line, sub {
		 printSelectionList("cat", $in{cat}, $bigCategoryPairs);
	       });
	}
      elsif ($in{cat} eq "ELE") {
	push(@line, $text{userTable_catEle});
      } else {
	push(@line, sub {
	  printSelectionList("cat", $in{cat}, $smallCategoryPairs);
	});
      }
    };

    push(@line, {align=>'right'});
    if(defined $params{lightweight}) {
      push(@line, "");
    } else {
      push(@line, $text{userdb_login});
    }
    push(@line, {align=>'left'});
    if ($params{loginInvariant}) {
      push(@line, "<font size=+2><b>$in{login}</b></font>");
    } elsif(defined $params{lightweight}) {
      push(@line, "");
    } else {
      push(@line, sub {printTextLine("login", $in{login},
				     {size => 8, maxlength => 8 })});
    };

    push(@bigForm, [ @line ]);

  if (!defined $params{lightweight}) {
    push(@bigForm,
	 sub { # 2e ligne
	   if(defined $params{batchUsers}) {
	     printLabeledTextLine("$text{userTable_comment}:",
				  "lastName",
				  $in{lastName},{size=>20});
	   } else {
	     printLabeledTextLine("$text{userTable_lastName}:",
				  "lastName",
				  $in{lastName},{size=>20});
	     printLabeledTextLine("$text{userTable_firstName}:",
				  "firstName",
				  $in{firstName},{size=>20});
	   }
	 });
  }

    push(@bigForm,
	 sub {
	   # 3e ligne
	   if (!defined $params{lightweight}) {
	     if (!defined ($in{"date.J"})) {
	       timeToJma(time, \%in);
	     }
	     printLabel($text{userTable_date}),
	     tableCell {&inputDate("date", \%in)} { nowrap=>1};
	   }

	   if ($in{cat} eq "ELE" && !$params{allCategories}) {
	     printLabel($text{userdb_class});
	     tableCell {
	       printClassWidget("classe",'class.'.$in{classe}, 0);
	     };
	   } else {
	     my $sponsor = $in{'classe'};

	     if($params{batchUsers} ne "EXA") {
	       printLabeledTextLine("$text{userdb_sponsor}:",
				    "classe", $sponsor, {size=>20});
	     } else {
	       tableCell {};
	       tableCell {};
	     }
	   }

	   if (defined $params{lightweight}) {
	     printLabel(" ");
	     tableCell {};
	   }

	 });

    if(!defined $params{lightweight} &&
       (!defined $params{batchUsers} ||
	(!$params{loginInvariant} && $config{ssn}))) {
      push(@bigForm,
	   sub { # 4e ligne
	     if (defined $params{batchUsers}) {
	       printLabel(" ");
	       tableCell {};
	     } else {
	       printLabeledTextLine($text{userdb_alias}, "alias",
				    $in{'alias'}, {size=>20});
	     }

	     if (!$params{loginInvariant} && $config{ssn}) {
	       printLabeledTextLine($text{defFormat_ssn},
				    "ssn",
				    $in{'ssn'},
				    {size=>11, maxlength=>11});
	     }
	   });
    }

    if(! $params{noPasswd} ) {
      push(@bigForm,
	   [# 5e ligne

	    {align=>'right'},
	    $text{userTable_password},
	    {align=>'left'},
	    sub { printPasswordInput("pass1", $in{'pass1'}, {size=>10}) },

	    {align=>'right'},
	    "$text{userTable_password} ($text{userTable_forConfirmation})",
	    {align=>'left'},
	    sub { printPasswordInput("pass2", $in{'pass2'}, {size=>10}) }
	   ]);
    }

    push(@bigForm,
	 sub { # Submit
	   submitLine(7, $params{label});

	   if ($params{batchUsers}) {
	     hidden(batchUsers => $params{batchUsers});
	     hidden(cat => $params{batchUsers});
	   }
	   if ($params{regularize}) {
	     &hidden("regularize", "1");
	   }
	   if($in{cat} eq "ELE" && !$params{allCategories}) {
	     hidden(cat => 'ELE');
	   }
	   if ($params{loginInvariant}) {
	     &dumpLogin;
	   }
	 }
	);

    mainForm ($params{action}, $params{title},  \@bigForm,
	      { width=>"90%" }
	      );

    hr;
}



1;
